"""LangChain integration for HyperspaceDB."""

from langchain_hyperspace.vectorstores import HyperspaceVectorStore
from langchain_hyperspace.embeddings import YARLabsEmbeddings

__version__ = "0.1.0"
__all__ = ["HyperspaceVectorStore", "YARLabsEmbeddings"]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: hyperspace.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'hyperspace.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10hyperspace.proto\x12\nhyperspace\"0\n\x12ReplicationRequest\x12\x1a\n\x12last_logical_clock\x18\x01 \x01(\x04\"\xaa\x02\n\x0eReplicationLog\x12\x15\n\rlogical_clock\x18\x01 \x01(\x04\x12\x16\n\x0eorigin_node_id\x18\x02 \x01(\t\x12\x12\n\ncollection\x18\x03 \x01(\t\x12&\n\x06insert\x18\x04 \x01(\x0b\x32\x14.hyperspace.InsertOpH\x00\x12;\n\x11\x63reate_collection\x18\x05 \x01(\x0b\x32\x1e.hyperspace.CreateCollectionOpH\x00\x12;\n\x11\x64\x65lete_collection\x18\x06 \x01(\x0b\x32\x1e.hyperspace.DeleteCollectionOpH\x00\x12&\n\x06\x64\x65lete\x18\x07 \x01(\x0b\x32\x14.hyperspace.DeleteOpH\x00\x42\x0b\n\toperation\"\x9f\x02\n\x08InsertOp\x12\n\n\x02id\x18\x01 \x01(\r\x12\x0e\n\x06vector\x18\x02 \x03(\x01\x12\x34\n\x08metadata\x18\x03 \x03(\x0b\x32\".hyperspace.InsertOp.MetadataEntry\x12?\n\x0etyped_metadata\x18\x04 \x03(\x0b\x32\'.hyperspace.InsertOp.TypedMetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aO\n\x12TypedMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.hyperspace.MetadataValue:\x02\x38\x01\"7\n\x12\x43reateCollectionOp\x12\x11\n\tdimension\x18\x01 \x01(\r\x12\x0e\n\x06metric\x18\x02 \x01(\t\"\x14\n\x12\x44\x65leteCollectionOp\"\x16\n\x08\x44\x65leteOp\x12\n\n\x02id\x18\x01 \x01(\r\"@\n\x12QuantizationConfig\x12*\n\x04mode\x18\x01 \x01(\x0e\x32\x1c.hyperspace.QuantizationMode\"J\n\x17\x43reateCollectionRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tdimension\x18\x02 \x01(\r\x12\x0e\n\x06metric\x18\x03 \x01(\t\"\'\n\x17\x44\x65leteCollectionRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\".\n\x17ListCollectionsResponse\x12\x13\n\x0b\x63ollections\x18\x01 \x03(\t\"&\n\x16\x43ollectionStatsRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"c\n\x17\x43ollectionStatsResponse\x12\r\n\x05\x63ount\x18\x01 \x01(\x04\x12\x11\n\tdimension\x18\x02 \x01(\r\x12\x0e\n\x06metric\x18\x03 \x01(\t\x12\x16\n\x0eindexing_queue\x18\x04 \x01(\x04\"n\n\x13RebuildIndexRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x38\n\x0c\x66ilter_query\x18\x02 \x01(\x0b\x32\x1d.hyperspace.VacuumFilterQueryH\x00\x88\x01\x01\x42\x0f\n\r_filter_query\"z\n\x0c\x43onfigUpdate\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x16\n\tef_search\x18\x02 \x01(\rH\x00\x88\x01\x01\x12\x1c\n\x0f\x65\x66_construction\x18\x03 \x01(\rH\x01\x88\x01\x01\x42\x0c\n\n_ef_searchB\x12\n\x10_ef_construction\";\n\x11VacuumFilterQuery\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\n\n\x02op\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\x01\"Z\n\x16ReconsolidationRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x15\n\rtarget_vector\x18\x02 \x03(\x01\x12\x15\n\rlearning_rate\x18\x03 \x01(\x01\"\xa2\x03\n\rInsertRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x0e\n\x06vector\x18\x02 \x03(\x01\x12\n\n\x02id\x18\x03 \x01(\r\x12\x39\n\x08metadata\x18\x04 \x03(\x0b\x32\'.hyperspace.InsertRequest.MetadataEntry\x12\x16\n\x0eorigin_node_id\x18\x05 \x01(\t\x12\x15\n\rlogical_clock\x18\x06 \x01(\x04\x12/\n\ndurability\x18\x07 \x01(\x0e\x32\x1b.hyperspace.DurabilityLevel\x12\x44\n\x0etyped_metadata\x18\x08 \x03(\x0b\x32,.hyperspace.InsertRequest.TypedMetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aO\n\x12TypedMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.hyperspace.MetadataValue:\x02\x38\x01\"\xa5\x02\n\nVectorData\x12\x0e\n\x06vector\x18\x01 \x03(\x01\x12\n\n\x02id\x18\x02 \x01(\r\x12\x36\n\x08metadata\x18\x03 \x03(\x0b\x32$.hyperspace.VectorData.MetadataEntry\x12\x41\n\x0etyped_metadata\x18\x04 \x03(\x0b\x32).hyperspace.VectorData.TypedMetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aO\n\x12TypedMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.hyperspace.MetadataValue:\x02\x38\x01\"\xb1\x01\n\x12\x42\x61tchInsertRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\'\n\x07vectors\x18\x02 \x03(\x0b\x32\x16.hyperspace.VectorData\x12\x16\n\x0eorigin_node_id\x18\x03 \x01(\t\x12\x15\n\rlogical_clock\x18\x04 \x01(\x04\x12/\n\ndurability\x18\x05 \x01(\x0e\x32\x1b.hyperspace.DurabilityLevel\"\xe2\x01\n\x11InsertTextRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\r\x12\x0c\n\x04text\x18\x03 \x01(\t\x12=\n\x08metadata\x18\x04 \x03(\x0b\x32+.hyperspace.InsertTextRequest.MetadataEntry\x12/\n\ndurability\x18\x05 \x01(\x0e\x32\x1b.hyperspace.DurabilityLevel\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"0\n\x10VectorizeRequest\x12\x0c\n\x04text\x18\x01 \x01(\t\x12\x0e\n\x06metric\x18\x02 \x01(\t\"#\n\x11VectorizeResponse\x12\x0e\n\x06vector\x18\x01 \x03(\x01\"\xd3\x01\n\x11SearchTextRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\x12\r\n\x05top_k\x18\x03 \x01(\r\x12\x39\n\x06\x66ilter\x18\x04 \x03(\x0b\x32).hyperspace.SearchTextRequest.FilterEntry\x12#\n\x07\x66ilters\x18\x05 \x03(\x0b\x32\x12.hyperspace.Filter\x1a-\n\x0b\x46ilterEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"!\n\x0eInsertResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"/\n\rDeleteRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\r\"!\n\x0e\x44\x65leteResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"\xbe\x02\n\rSearchRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x0e\n\x06vector\x18\x02 \x03(\x01\x12\r\n\x05top_k\x18\x03 \x01(\r\x12\x35\n\x06\x66ilter\x18\x04 \x03(\x0b\x32%.hyperspace.SearchRequest.FilterEntry\x12#\n\x07\x66ilters\x18\x05 \x03(\x0b\x32\x12.hyperspace.Filter\x12\x19\n\x0chybrid_query\x18\x06 \x01(\tH\x00\x88\x01\x01\x12\x19\n\x0chybrid_alpha\x18\x07 \x01(\x02H\x01\x88\x01\x01\x12\x17\n\x0fuse_wasserstein\x18\x08 \x01(\x08\x1a-\n\x0b\x46ilterEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x0f\n\r_hybrid_queryB\x0f\n\r_hybrid_alpha\"]\n\x06\x46ilter\x12\"\n\x05match\x18\x01 \x01(\x0b\x32\x11.hyperspace.MatchH\x00\x12\"\n\x05range\x18\x02 \x01(\x0b\x32\x11.hyperspace.RangeH\x00\x42\x0b\n\tcondition\"#\n\x05Match\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\x8c\x01\n\x05Range\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x10\n\x03gte\x18\x02 \x01(\x03H\x00\x88\x01\x01\x12\x10\n\x03lte\x18\x03 \x01(\x03H\x01\x88\x01\x01\x12\x14\n\x07gte_f64\x18\x04 \x01(\x01H\x02\x88\x01\x01\x12\x14\n\x07lte_f64\x18\x05 \x01(\x01H\x03\x88\x01\x01\x42\x06\n\x04_gteB\x06\n\x04_lteB\n\n\x08_gte_f64B\n\n\x08_lte_f64\";\n\x0eSearchResponse\x12)\n\x07results\x18\x01 \x03(\x0b\x32\x18.hyperspace.SearchResult\"A\n\x12\x42\x61tchSearchRequest\x12+\n\x08searches\x18\x01 \x03(\x0b\x32\x19.hyperspace.SearchRequest\"D\n\x13\x42\x61tchSearchResponse\x12-\n\tresponses\x18\x01 \x03(\x0b\x32\x1a.hyperspace.SearchResponse\"R\n\x1cSearchMultiCollectionRequest\x12\x13\n\x0b\x63ollections\x18\x01 \x03(\t\x12\x0e\n\x06vector\x18\x02 \x03(\x01\x12\r\n\x05top_k\x18\x03 \x01(\r\"\xba\x01\n\x1dSearchMultiCollectionResponse\x12K\n\tresponses\x18\x01 \x03(\x0b\x32\x38.hyperspace.SearchMultiCollectionResponse.ResponsesEntry\x1aL\n\x0eResponsesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.hyperspace.SearchResponse:\x02\x38\x01\"\xad\x02\n\x0cSearchResult\x12\n\n\x02id\x18\x01 \x01(\r\x12\x10\n\x08\x64istance\x18\x02 \x01(\x01\x12\x38\n\x08metadata\x18\x03 \x03(\x0b\x32&.hyperspace.SearchResult.MetadataEntry\x12\x43\n\x0etyped_metadata\x18\x04 \x03(\x0b\x32+.hyperspace.SearchResult.TypedMetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aO\n\x12TypedMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.hyperspace.MetadataValue:\x02\x38\x01\"?\n\x0eGetNodeRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\r\x12\r\n\x05layer\x18\x03 \x01(\r\"\xb4\x02\n\tGraphNode\x12\n\n\x02id\x18\x01 \x01(\r\x12\r\n\x05layer\x18\x02 \x01(\r\x12\x11\n\tneighbors\x18\x03 \x03(\r\x12\x35\n\x08metadata\x18\x04 \x03(\x0b\x32#.hyperspace.GraphNode.MetadataEntry\x12@\n\x0etyped_metadata\x18\x05 \x03(\x0b\x32(.hyperspace.GraphNode.TypedMetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aO\n\x12TypedMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.hyperspace.MetadataValue:\x02\x38\x01\"c\n\x13GetNeighborsRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\r\x12\r\n\x05layer\x18\x03 \x01(\r\x12\r\n\x05limit\x18\x04 \x01(\r\x12\x0e\n\x06offset\x18\x05 \x01(\r\"V\n\x14GetNeighborsResponse\x12(\n\tneighbors\x18\x01 \x03(\x0b\x32\x15.hyperspace.GraphNode\x12\x14\n\x0c\x65\x64ge_weights\x18\x02 \x03(\x01\"\xf9\x01\n\x0fTraverseRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x10\n\x08start_id\x18\x02 \x01(\r\x12\x11\n\tmax_depth\x18\x03 \x01(\r\x12\x11\n\tmax_nodes\x18\x04 \x01(\r\x12\r\n\x05layer\x18\x05 \x01(\r\x12\x37\n\x06\x66ilter\x18\x06 \x03(\x0b\x32\'.hyperspace.TraverseRequest.FilterEntry\x12#\n\x07\x66ilters\x18\x07 \x03(\x0b\x32\x12.hyperspace.Filter\x1a-\n\x0b\x46ilterEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"8\n\x10TraverseResponse\x12$\n\x05nodes\x18\x01 \x03(\x0b\x32\x15.hyperspace.GraphNode\"\x83\x01\n\x1b\x46indSemanticClustersRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\r\n\x05layer\x18\x02 \x01(\r\x12\x18\n\x10min_cluster_size\x18\x03 \x01(\r\x12\x14\n\x0cmax_clusters\x18\x04 \x01(\r\x12\x11\n\tmax_nodes\x18\x05 \x01(\r\"X\n\x18GetConceptParentsRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\r\x12\r\n\x05layer\x18\x03 \x01(\r\x12\r\n\x05limit\x18\x04 \x01(\r\"C\n\x19GetConceptParentsResponse\x12&\n\x07parents\x18\x01 \x03(\x0b\x32\x15.hyperspace.GraphNode\" \n\x0cGraphCluster\x12\x10\n\x08node_ids\x18\x01 \x03(\r\"J\n\x1c\x46indSemanticClustersResponse\x12*\n\x08\x63lusters\x18\x01 \x03(\x0b\x32\x18.hyperspace.GraphCluster\"r\n\rMetadataValue\x12\x16\n\x0cstring_value\x18\x01 \x01(\tH\x00\x12\x13\n\tint_value\x18\x02 \x01(\x03H\x00\x12\x16\n\x0c\x64ouble_value\x18\x03 \x01(\x01H\x00\x12\x14\n\nbool_value\x18\x04 \x01(\x08H\x00\x42\x06\n\x04kind\"h\n\x18\x45ventSubscriptionRequest\x12$\n\x05types\x18\x01 \x03(\x0e\x32\x15.hyperspace.EventType\x12\x17\n\ncollection\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\r\n\x0b_collection\"\xf3\x02\n\x13VectorInsertedEvent\x12\n\n\x02id\x18\x01 \x01(\r\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x15\n\rlogical_clock\x18\x03 \x01(\x04\x12\x16\n\x0eorigin_node_id\x18\x04 \x01(\t\x12?\n\x08metadata\x18\x05 \x03(\x0b\x32-.hyperspace.VectorInsertedEvent.MetadataEntry\x12J\n\x0etyped_metadata\x18\x06 \x03(\x0b\x32\x32.hyperspace.VectorInsertedEvent.TypedMetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aO\n\x12TypedMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.hyperspace.MetadataValue:\x02\x38\x01\"c\n\x12VectorDeletedEvent\x12\n\n\x02id\x18\x01 \x01(\r\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x15\n\rlogical_clock\x18\x03 \x01(\x04\x12\x16\n\x0eorigin_node_id\x18\x04 \x01(\t\"\xb4\x01\n\x0c\x45ventMessage\x12#\n\x04type\x18\x01 \x01(\x0e\x32\x15.hyperspace.EventType\x12:\n\x0fvector_inserted\x18\x02 \x01(\x0b\x32\x1f.hyperspace.VectorInsertedEventH\x00\x12\x38\n\x0evector_deleted\x18\x03 \x01(\x0b\x32\x1e.hyperspace.VectorDeletedEventH\x00\x42\t\n\x07payload\"\x07\n\x05\x45mpty\" \n\x0eStatusResponse\x12\x0e\n\x06status\x18\x01 \x01(\t\"\x10\n\x0eMonitorRequest\"e\n\x0bSystemStats\x12\x19\n\x11total_collections\x18\x01 \x01(\x04\x12\x15\n\rtotal_vectors\x18\x02 \x01(\x04\x12\x17\n\x0ftotal_memory_mb\x18\x03 \x01(\x01\x12\x0b\n\x03qps\x18\x04 \x01(\x01\"#\n\rDigestRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\"[\n\x0e\x44igestResponse\x12\x15\n\rlogical_clock\x18\x01 \x01(\x04\x12\x12\n\nstate_hash\x18\x02 \x01(\x04\x12\x0f\n\x07\x62uckets\x18\x03 \x03(\x04\x12\r\n\x05\x63ount\x18\x04 \x01(\x04\"v\n\x14SyncHandshakeRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x16\n\x0e\x63lient_buckets\x18\x02 \x03(\x04\x12\x1c\n\x14\x63lient_logical_clock\x18\x03 \x01(\x04\x12\x14\n\x0c\x63lient_count\x18\x04 \x01(\x04\"L\n\nDiffBucket\x12\x14\n\x0c\x62ucket_index\x18\x01 \x01(\r\x12\x13\n\x0bserver_hash\x18\x02 \x01(\x04\x12\x13\n\x0b\x63lient_hash\x18\x03 \x01(\x04\"\x8a\x01\n\x15SyncHandshakeResponse\x12,\n\x0c\x64iff_buckets\x18\x01 \x03(\x0b\x32\x16.hyperspace.DiffBucket\x12\x1c\n\x14server_logical_clock\x18\x02 \x01(\x04\x12\x14\n\x0cserver_count\x18\x03 \x01(\x04\x12\x0f\n\x07in_sync\x18\x04 \x01(\x08\"=\n\x0fSyncPullRequest\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x16\n\x0e\x62ucket_indices\x18\x02 \x03(\r\"\xc3\x01\n\x0eSyncVectorData\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\r\x12\x0e\n\x06vector\x18\x03 \x03(\x01\x12:\n\x08metadata\x18\x04 \x03(\x0b\x32(.hyperspace.SyncVectorData.MetadataEntry\x12\x14\n\x0c\x62ucket_index\x18\x05 \x01(\r\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"J\n\x10SyncPushResponse\x12\x10\n\x08\x61\x63\x63\x65pted\x18\x01 \x01(\r\x12\x10\n\x08rejected\x18\x02 \x01(\r\x12\x12\n\nduplicates\x18\x03 \x01(\r*+\n\x10QuantizationMode\x12\x08\n\x04NONE\x10\x00\x12\r\n\tSCALAR_I8\x10\x01*F\n\x0f\x44urabilityLevel\x12\x11\n\rDEFAULT_LEVEL\x10\x00\x12\t\n\x05\x41SYNC\x10\x01\x12\t\n\x05\x42\x41TCH\x10\x02\x12\n\n\x06STRICT\x10\x03*G\n\tEventType\x12\x11\n\rEVENT_UNKNOWN\x10\x00\x12\x13\n\x0fVECTOR_INSERTED\x10\x01\x12\x12\n\x0eVECTOR_DELETED\x10\x02\x32\xa4\x12\n\x08\x44\x61tabase\x12S\n\x10\x43reateCollection\x12#.hyperspace.CreateCollectionRequest\x1a\x1a.hyperspace.StatusResponse\x12S\n\x10\x44\x65leteCollection\x12#.hyperspace.DeleteCollectionRequest\x1a\x1a.hyperspace.StatusResponse\x12I\n\x0fListCollections\x12\x11.hyperspace.Empty\x1a#.hyperspace.ListCollectionsResponse\x12]\n\x12GetCollectionStats\x12\".hyperspace.CollectionStatsRequest\x1a#.hyperspace.CollectionStatsResponse\x12?\n\x06Insert\x12\x19.hyperspace.InsertRequest\x1a\x1a.hyperspace.InsertResponse\x12I\n\x0b\x42\x61tchInsert\x12\x1e.hyperspace.BatchInsertRequest\x1a\x1a.hyperspace.InsertResponse\x12G\n\nInsertText\x12\x1d.hyperspace.InsertTextRequest\x1a\x1a.hyperspace.InsertResponse\x12H\n\tVectorize\x12\x1c.hyperspace.VectorizeRequest\x1a\x1d.hyperspace.VectorizeResponse\x12G\n\nSearchText\x12\x1d.hyperspace.SearchTextRequest\x1a\x1a.hyperspace.SearchResponse\x12?\n\x06\x44\x65lete\x12\x19.hyperspace.DeleteRequest\x1a\x1a.hyperspace.DeleteResponse\x12?\n\x06Search\x12\x19.hyperspace.SearchRequest\x1a\x1a.hyperspace.SearchResponse\x12N\n\x0bSearchBatch\x12\x1e.hyperspace.BatchSearchRequest\x1a\x1f.hyperspace.BatchSearchResponse\x12l\n\x15SearchMultiCollection\x12(.hyperspace.SearchMultiCollectionRequest\x1a).hyperspace.SearchMultiCollectionResponse\x12<\n\x07GetNode\x12\x1a.hyperspace.GetNodeRequest\x1a\x15.hyperspace.GraphNode\x12Q\n\x0cGetNeighbors\x12\x1f.hyperspace.GetNeighborsRequest\x1a .hyperspace.GetNeighborsResponse\x12`\n\x11GetConceptParents\x12$.hyperspace.GetConceptParentsRequest\x1a%.hyperspace.GetConceptParentsResponse\x12\x45\n\x08Traverse\x12\x1b.hyperspace.TraverseRequest\x1a\x1c.hyperspace.TraverseResponse\x12i\n\x14\x46indSemanticClusters\x12\'.hyperspace.FindSemanticClustersRequest\x1a(.hyperspace.FindSemanticClustersResponse\x12@\n\x07Monitor\x12\x1a.hyperspace.MonitorRequest\x1a\x17.hyperspace.SystemStats0\x01\x12@\n\x0fTriggerSnapshot\x12\x11.hyperspace.Empty\x1a\x1a.hyperspace.StatusResponse\x12>\n\rTriggerVacuum\x12\x11.hyperspace.Empty\x1a\x1a.hyperspace.StatusResponse\x12X\n\x16TriggerReconsolidation\x12\".hyperspace.ReconsolidationRequest\x1a\x1a.hyperspace.StatusResponse\x12\x41\n\tConfigure\x12\x18.hyperspace.ConfigUpdate\x1a\x1a.hyperspace.StatusResponse\x12I\n\tReplicate\x12\x1e.hyperspace.ReplicationRequest\x1a\x1a.hyperspace.ReplicationLog0\x01\x12U\n\x11SubscribeToEvents\x12$.hyperspace.EventSubscriptionRequest\x1a\x18.hyperspace.EventMessage0\x01\x12\x42\n\tGetDigest\x12\x19.hyperspace.DigestRequest\x1a\x1a.hyperspace.DigestResponse\x12K\n\x0cRebuildIndex\x12\x1f.hyperspace.RebuildIndexRequest\x1a\x1a.hyperspace.StatusResponse\x12T\n\rSyncHandshake\x12 .hyperspace.SyncHandshakeRequest\x1a!.hyperspace.SyncHandshakeResponse\x12\x45\n\x08SyncPull\x12\x1b.hyperspace.SyncPullRequest\x1a\x1a.hyperspace.SyncVectorData0\x01\x12\x46\n\x08SyncPush\x12\x1a.hyperspace.SyncVectorData\x1a\x1c.hyperspace.SyncPushResponse(\x01\x42,Z*github.com/yarlabs/hyperspace-sdk-go/protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'hyperspace_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z*github.com/yarlabs/hyperspace-sdk-go/proto'
  _globals['_INSERTOP_METADATAENTRY']._loaded_options = None
  _globals['_INSERTOP_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_INSERTOP_TYPEDMETADATAENTRY']._loaded_options = None
  _globals['_INSERTOP_TYPEDMETADATAENTRY']._serialized_options = b'8\001'
  _globals['_INSERTREQUEST_METADATAENTRY']._loaded_options = None
  _globals['_INSERTREQUEST_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_INSERTREQUEST_TYPEDMETADATAENTRY']._loaded_options = None
  _globals['_INSERTREQUEST_TYPEDMETADATAENTRY']._serialized_options = b'8\001'
  _globals['_VECTORDATA_METADATAENTRY']._loaded_options = None
  _globals['_VECTORDATA_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_VECTORDATA_TYPEDMETADATAENTRY']._loaded_options = None
  _globals['_VECTORDATA_TYPEDMETADATAENTRY']._serialized_options = b'8\001'
  _globals['_INSERTTEXTREQUEST_METADATAENTRY']._loaded_options = None
  _globals['_INSERTTEXTREQUEST_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_SEARCHTEXTREQUEST_FILTERENTRY']._loaded_options = None
  _globals['_SEARCHTEXTREQUEST_FILTERENTRY']._serialized_options = b'8\001'
  _globals['_SEARCHREQUEST_FILTERENTRY']._loaded_options = None
  _globals['_SEARCHREQUEST_FILTERENTRY']._serialized_options = b'8\001'
  _globals['_SEARCHMULTICOLLECTIONRESPONSE_RESPONSESENTRY']._loaded_options = None
  _globals['_SEARCHMULTICOLLECTIONRESPONSE_RESPONSESENTRY']._serialized_options = b'8\001'
  _globals['_SEARCHRESULT_METADATAENTRY']._loaded_options = None
  _globals['_SEARCHRESULT_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_SEARCHRESULT_TYPEDMETADATAENTRY']._loaded_options = None
  _globals['_SEARCHRESULT_TYPEDMETADATAENTRY']._serialized_options = b'8\001'
  _globals['_GRAPHNODE_METADATAENTRY']._loaded_options = None
  _globals['_GRAPHNODE_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_GRAPHNODE_TYPEDMETADATAENTRY']._loaded_options = None
  _globals['_GRAPHNODE_TYPEDMETADATAENTRY']._serialized_options = b'8\001'
  _globals['_TRAVERSEREQUEST_FILTERENTRY']._loaded_options = None
  _globals['_TRAVERSEREQUEST_FILTERENTRY']._serialized_options = b'8\001'
  _globals['_VECTORINSERTEDEVENT_METADATAENTRY']._loaded_options = None
  _globals['_VECTORINSERTEDEVENT_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_VECTORINSERTEDEVENT_TYPEDMETADATAENTRY']._loaded_options = None
  _globals['_VECTORINSERTEDEVENT_TYPEDMETADATAENTRY']._serialized_options = b'8\001'
  _globals['_SYNCVECTORDATA_METADATAENTRY']._loaded_options = None
  _globals['_SYNCVECTORDATA_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_QUANTIZATIONMODE']._serialized_start=7582
  _globals['_QUANTIZATIONMODE']._serialized_end=7625
  _globals['_DURABILITYLEVEL']._serialized_start=7627
  _globals['_DURABILITYLEVEL']._serialized_end=7697
  _globals['_EVENTTYPE']._serialized_start=7699
  _globals['_EVENTTYPE']._serialized_end=7770
  _globals['_REPLICATIONREQUEST']._serialized_start=32
  _globals['_REPLICATIONREQUEST']._serialized_end=80
  _globals['_REPLICATIONLOG']._serialized_start=83
  _globals['_REPLICATIONLOG']._serialized_end=381
  _globals['_INSERTOP']._serialized_start=384
  _globals['_INSERTOP']._serialized_end=671
  _globals['_INSERTOP_METADATAENTRY']._serialized_start=543
  _globals['_INSERTOP_METADATAENTRY']._serialized_end=590
  _globals['_INSERTOP_TYPEDMETADATAENTRY']._serialized_start=592
  _globals['_INSERTOP_TYPEDMETADATAENTRY']._serialized_end=671
  _globals['_CREATECOLLECTIONOP']._serialized_start=673
  _globals['_CREATECOLLECTIONOP']._serialized_end=728
  _globals['_DELETECOLLECTIONOP']._serialized_start=730
  _globals['_DELETECOLLECTIONOP']._serialized_end=750
  _globals['_DELETEOP']._serialized_start=752
  _globals['_DELETEOP']._serialized_end=774
  _globals['_QUANTIZATIONCONFIG']._serialized_start=776
  _globals['_QUANTIZATIONCONFIG']._serialized_end=840
  _globals['_CREATECOLLECTIONREQUEST']._serialized_start=842
  _globals['_CREATECOLLECTIONREQUEST']._serialized_end=916
  _globals['_DELETECOLLECTIONREQUEST']._serialized_start=918
  _globals['_DELETECOLLECTIONREQUEST']._serialized_end=957
  _globals['_LISTCOLLECTIONSRESPONSE']._serialized_start=959
  _globals['_LISTCOLLECTIONSRESPONSE']._serialized_end=1005
  _globals['_COLLECTIONSTATSREQUEST']._serialized_start=1007
  _globals['_COLLECTIONSTATSREQUEST']._serialized_end=1045
  _globals['_COLLECTIONSTATSRESPONSE']._serialized_start=1047
  _globals['_COLLECTIONSTATSRESPONSE']._serialized_end=1146
  _globals['_REBUILDINDEXREQUEST']._serialized_start=1148
  _globals['_REBUILDINDEXREQUEST']._serialized_end=1258
  _globals['_CONFIGUPDATE']._serialized_start=1260
  _globals['_CONFIGUPDATE']._serialized_end=1382
  _globals['_VACUUMFILTERQUERY']._serialized_start=1384
  _globals['_VACUUMFILTERQUERY']._serialized_end=1443
  _globals['_RECONSOLIDATIONREQUEST']._serialized_start=1445
  _globals['_RECONSOLIDATIONREQUEST']._serialized_end=1535
  _globals['_INSERTREQUEST']._serialized_start=1538
  _globals['_INSERTREQUEST']._serialized_end=1956
  _globals['_INSERTREQUEST_METADATAENTRY']._serialized_start=543
  _globals['_INSERTREQUEST_METADATAENTRY']._serialized_end=590
  _globals['_INSERTREQUEST_TYPEDMETADATAENTRY']._serialized_start=592
  _globals['_INSERTREQUEST_TYPEDMETADATAENTRY']._serialized_end=671
  _globals['_VECTORDATA']._serialized_start=1959
  _globals['_VECTORDATA']._serialized_end=2252
  _globals['_VECTORDATA_METADATAENTRY']._serialized_start=543
  _globals['_VECTORDATA_METADATAENTRY']._serialized_end=590
  _globals['_VECTORDATA_TYPEDMETADATAENTRY']._serialized_start=592
  _globals['_VECTORDATA_TYPEDMETADATAENTRY']._serialized_end=671
  _globals['_BATCHINSERTREQUEST']._serialized_start=2255
  _globals['_BATCHINSERTREQUEST']._serialized_end=2432
  _globals['_INSERTTEXTREQUEST']._serialized_start=2435
  _globals['_INSERTTEXTREQUEST']._serialized_end=2661
  _globals['_INSERTTEXTREQUEST_METADATAENTRY']._serialized_start=543
  _globals['_INSERTTEXTREQUEST_METADATAENTRY']._serialized_end=590
  _globals['_VECTORIZEREQUEST']._serialized_start=2663
  _globals['_VECTORIZEREQUEST']._serialized_end=2711
  _globals['_VECTORIZERESPONSE']._serialized_start=2713
  _globals['_VECTORIZERESPONSE']._serialized_end=2748
  _globals['_SEARCHTEXTREQUEST']._serialized_start=2751
  _globals['_SEARCHTEXTREQUEST']._serialized_end=2962
  _globals['_SEARCHTEXTREQUEST_FILTERENTRY']._serialized_start=2917
  _globals['_SEARCHTEXTREQUEST_FILTERENTRY']._serialized_end=2962
  _globals['_INSERTRESPONSE']._serialized_start=2964
  _globals['_INSERTRESPONSE']._serialized_end=2997
  _globals['_DELETEREQUEST']._serialized_start=2999
  _globals['_DELETEREQUEST']._serialized_end=3046
  _globals['_DELETERESPONSE']._serialized_start=3048
  _globals['_DELETERESPONSE']._serialized_end=3081
  _globals['_SEARCHREQUEST']._serialized_start=3084
  _globals['_SEARCHREQUEST']._serialized_end=3402
  _globals['_SEARCHREQUEST_FILTERENTRY']._serialized_start=2917
  _globals['_SEARCHREQUEST_FILTERENTRY']._serialized_end=2962
  _globals['_FILTER']._serialized_start=3404
  _globals['_FILTER']._serialized_end=3497
  _globals['_MATCH']._serialized_start=3499
  _globals['_MATCH']._serialized_end=3534
  _globals['_RANGE']._serialized_start=3537
  _globals['_RANGE']._serialized_end=3677
  _globals['_SEARCHRESPONSE']._serialized_start=3679
  _globals['_SEARCHRESPONSE']._serialized_end=3738
  _globals['_BATCHSEARCHREQUEST']._serialized_start=3740
  _globals['_BATCHSEARCHREQUEST']._serialized_end=3805
  _globals['_BATCHSEARCHRESPONSE']._serialized_start=3807
  _globals['_BATCHSEARCHRESPONSE']._serialized_end=3875
  _globals['_SEARCHMULTICOLLECTIONREQUEST']._serialized_start=3877
  _globals['_SEARCHMULTICOLLECTIONREQUEST']._serialized_end=3959
  _globals['_SEARCHMULTICOLLECTIONRESPONSE']._serialized_start=3962
  _globals['_SEARCHMULTICOLLECTIONRESPONSE']._serialized_end=4148
  _globals['_SEARCHMULTICOLLECTIONRESPONSE_RESPONSESENTRY']._serialized_start=4072
  _globals['_SEARCHMULTICOLLECTIONRESPONSE_RESPONSESENTRY']._serialized_end=4148
  _globals['_SEARCHRESULT']._serialized_start=4151
  _globals['_SEARCHRESULT']._serialized_end=4452
  _globals['_SEARCHRESULT_METADATAENTRY']._serialized_start=543
  _globals['_SEARCHRESULT_METADATAENTRY']._serialized_end=590
  _globals['_SEARCHRESULT_TYPEDMETADATAENTRY']._serialized_start=592
  _globals['_SEARCHRESULT_TYPEDMETADATAENTRY']._serialized_end=671
  _globals['_GETNODEREQUEST']._serialized_start=4454
  _globals['_GETNODEREQUEST']._serialized_end=4517
  _globals['_GRAPHNODE']._serialized_start=4520
  _globals['_GRAPHNODE']._serialized_end=4828
  _globals['_GRAPHNODE_METADATAENTRY']._serialized_start=543
  _globals['_GRAPHNODE_METADATAENTRY']._serialized_end=590
  _globals['_GRAPHNODE_TYPEDMETADATAENTRY']._serialized_start=592
  _globals['_GRAPHNODE_TYPEDMETADATAENTRY']._serialized_end=671
  _globals['_GETNEIGHBORSREQUEST']._serialized_start=4830
  _globals['_GETNEIGHBORSREQUEST']._serialized_end=4929
  _globals['_GETNEIGHBORSRESPONSE']._serialized_start=4931
  _globals['_GETNEIGHBORSRESPONSE']._serialized_end=5017
  _globals['_TRAVERSEREQUEST']._serialized_start=5020
  _globals['_TRAVERSEREQUEST']._serialized_end=5269
  _globals['_TRAVERSEREQUEST_FILTERENTRY']._serialized_start=2917
  _globals['_TRAVERSEREQUEST_FILTERENTRY']._serialized_end=2962
  _globals['_TRAVERSERESPONSE']._serialized_start=5271
  _globals['_TRAVERSERESPONSE']._serialized_end=5327
  _globals['_FINDSEMANTICCLUSTERSREQUEST']._serialized_start=5330
  _globals['_FINDSEMANTICCLUSTERSREQUEST']._serialized_end=5461
  _globals['_GETCONCEPTPARENTSREQUEST']._serialized_start=5463
  _globals['_GETCONCEPTPARENTSREQUEST']._serialized_end=5551
  _globals['_GETCONCEPTPARENTSRESPONSE']._serialized_start=5553
  _globals['_GETCONCEPTPARENTSRESPONSE']._serialized_end=5620
  _globals['_GRAPHCLUSTER']._serialized_start=5622
  _globals['_GRAPHCLUSTER']._serialized_end=5654
  _globals['_FINDSEMANTICCLUSTERSRESPONSE']._serialized_start=5656
  _globals['_FINDSEMANTICCLUSTERSRESPONSE']._serialized_end=5730
  _globals['_METADATAVALUE']._serialized_start=5732
  _globals['_METADATAVALUE']._serialized_end=5846
  _globals['_EVENTSUBSCRIPTIONREQUEST']._serialized_start=5848
  _globals['_EVENTSUBSCRIPTIONREQUEST']._serialized_end=5952
  _globals['_VECTORINSERTEDEVENT']._serialized_start=5955
  _globals['_VECTORINSERTEDEVENT']._serialized_end=6326
  _globals['_VECTORINSERTEDEVENT_METADATAENTRY']._serialized_start=543
  _globals['_VECTORINSERTEDEVENT_METADATAENTRY']._serialized_end=590
  _globals['_VECTORINSERTEDEVENT_TYPEDMETADATAENTRY']._serialized_start=592
  _globals['_VECTORINSERTEDEVENT_TYPEDMETADATAENTRY']._serialized_end=671
  _globals['_VECTORDELETEDEVENT']._serialized_start=6328
  _globals['_VECTORDELETEDEVENT']._serialized_end=6427
  _globals['_EVENTMESSAGE']._serialized_start=6430
  _globals['_EVENTMESSAGE']._serialized_end=6610
  _globals['_EMPTY']._serialized_start=6612
  _globals['_EMPTY']._serialized_end=6619
  _globals['_STATUSRESPONSE']._serialized_start=6621
  _globals['_STATUSRESPONSE']._serialized_end=6653
  _globals['_MONITORREQUEST']._serialized_start=6655
  _globals['_MONITORREQUEST']._serialized_end=6671
  _globals['_SYSTEMSTATS']._serialized_start=6673
  _globals['_SYSTEMSTATS']._serialized_end=6774
  _globals['_DIGESTREQUEST']._serialized_start=6776
  _globals['_DIGESTREQUEST']._serialized_end=6811
  _globals['_DIGESTRESPONSE']._serialized_start=6813
  _globals['_DIGESTRESPONSE']._serialized_end=6904
  _globals['_SYNCHANDSHAKEREQUEST']._serialized_start=6906
  _globals['_SYNCHANDSHAKEREQUEST']._serialized_end=7024
  _globals['_DIFFBUCKET']._serialized_start=7026
  _globals['_DIFFBUCKET']._serialized_end=7102
  _globals['_SYNCHANDSHAKERESPONSE']._serialized_start=7105
  _globals['_SYNCHANDSHAKERESPONSE']._serialized_end=7243
  _globals['_SYNCPULLREQUEST']._serialized_start=7245
  _globals['_SYNCPULLREQUEST']._serialized_end=7306
  _globals['_SYNCVECTORDATA']._serialized_start=7309
  _globals['_SYNCVECTORDATA']._serialized_end=7504
  _globals['_SYNCVECTORDATA_METADATAENTRY']._serialized_start=543
  _globals['_SYNCVECTORDATA_METADATAENTRY']._serialized_end=590
  _globals['_SYNCPUSHRESPONSE']._serialized_start=7506
  _globals['_SYNCPUSHRESPONSE']._serialized_end=7580
  _globals['_DATABASE']._serialized_start=7773
  _globals['_DATABASE']._serialized_end=10113
# @@protoc_insertion_point(module_scope)



"""Unit tests for HyperspaceVectorStore."""

import pytest
from unittest.mock import Mock, MagicMock, patch
from langchain_core.documents import Document

from langchain_hyperspace import HyperspaceVectorStore


class MockEmbeddings:
    """Mock embeddings for testing."""
    
    def embed_documents(self, texts):
        """Return mock embeddings for documents."""
        return [[0.1] * 1536 for _ in texts]
    
    def embed_query(self, text):
        """Return mock embedding for query."""
        return [0.1] * 1536


@pytest.fixture
def mock_embeddings():
    """Fixture for mock embeddings."""
    return MockEmbeddings()


@pytest.fixture
def vectorstore(mock_embeddings):
    """Fixture for HyperspaceVectorStore."""
    with patch('grpc.insecure_channel'):
        store = HyperspaceVectorStore(
            host="localhost",
            port=50051,
            collection_name="test_collection",
            embedding_function=mock_embeddings,
            dimension=1536,
        )
        return store


class TestHyperspaceVectorStore:
    """Test suite for HyperspaceVectorStore."""
    
    def test_initialization(self, vectorstore):
        """Test vector store initialization."""
        assert vectorstore.host == "localhost"
        assert vectorstore.port == 50051
        assert vectorstore.collection_name == "test_collection"
        assert vectorstore.dimension == 1536
        assert vectorstore.metric == "l2"
        assert vectorstore.enable_deduplication is True
    
    def test_content_hash_deterministic(self, vectorstore):
        """Test that content hashing is deterministic."""
        text = "Hello, world!"
        hash1 = vectorstore._compute_content_hash(text)
        hash2 = vectorstore._compute_content_hash(text)
        assert hash1 == hash2
    
    def test_content_hash_different(self, vectorstore):
        """Test that different content produces different hashes."""
        hash1 = vectorstore._compute_content_hash("Hello")
        hash2 = vectorstore._compute_content_hash("World")
        assert hash1 != hash2
    
    def test_add_texts_basic(self, vectorstore):
        """Test basic text addition."""
        texts = ["Hello", "World"]
        ids = vectorstore.add_texts(texts)
        
        assert len(ids) == 2
        assert all(isinstance(id, str) for id in ids)
    
    def test_add_texts_with_metadata(self, vectorstore):
        """Test adding texts with metadata."""
        texts = ["Hello", "World"]
        metadatas = [{"source": "test1"}, {"source": "test2"}]
        
        ids = vectorstore.add_texts(texts, metadatas=metadatas)
        assert len(ids) == 2
    
    def test_add_texts_deduplication(self, vectorstore):
        """Test that deduplication uses content hash."""
        texts = ["Same text", "Same text", "Different text"]
        ids = vectorstore.add_texts(texts)
        
        # First two should have same ID (content hash)
        assert ids[0] == ids[1]
        assert ids[0] != ids[2]
    
    def test_add_texts_no_deduplication(self, mock_embeddings):
        """Test adding texts without deduplication."""
        with patch('grpc.insecure_channel'):
            vectorstore = HyperspaceVectorStore(
                host="localhost",
                port=50051,
                collection_name="test",
                embedding_function=mock_embeddings,
                enable_deduplication=False,
            )
            
            texts = ["Same text", "Same text"]
            ids = vectorstore.add_texts(texts)
            
            # Should have different IDs
            assert ids[0] != ids[1]
    
    def test_add_texts_no_embedding_function(self):
        """Test that add_texts fails without embedding function."""
        with patch('grpc.insecure_channel'):
            vectorstore = HyperspaceVectorStore(
                host="localhost",
                port=50051,
                collection_name="test",
                embedding_function=None,
            )
            
            with pytest.raises(ValueError, match="embedding_function is required"):
                vectorstore.add_texts(["Hello"])
    
    def test_similarity_search_no_embedding_function(self):
        """Test that search fails without embedding function."""
        with patch('grpc.insecure_channel'):
            vectorstore = HyperspaceVectorStore(
                host="localhost",
                port=50051,
                collection_name="test",
                embedding_function=None,
            )
            
            with pytest.raises(ValueError, match="embedding_function is required"):
                vectorstore.similarity_search("query")
    
    def test_from_texts(self, mock_embeddings):
        """Test creating vector store from texts."""
        with patch('grpc.insecure_channel'):
            texts = ["Hello", "World"]
            vectorstore = HyperspaceVectorStore.from_texts(
                texts=texts,
                embedding=mock_embeddings,
                host="localhost",
                port=50051,
            )
            
            assert vectorstore.collection_name == "default"
    
    def test_from_documents(self, mock_embeddings):
        """Test creating vector store from documents."""
        with patch('grpc.insecure_channel'):
            docs = [
                Document(page_content="Hello", metadata={"source": "test1"}),
                Document(page_content="World", metadata={"source": "test2"}),
            ]
            
            vectorstore = HyperspaceVectorStore.from_documents(
                documents=docs,
                embedding=mock_embeddings,
                host="localhost",
                port=50051,
            )
            
            assert vectorstore.collection_name == "default"
    
    def test_metadata_with_api_key(self, mock_embeddings):
        """Test that API key is included in metadata."""
        with patch('grpc.insecure_channel'):
            vectorstore = HyperspaceVectorStore(
                host="localhost",
                port=50051,
                collection_name="test",
                embedding_function=mock_embeddings,
                api_key="test_key",
            )
            
            metadata = vectorstore._get_metadata()
            assert ("x-api-key", "test_key") in metadata
    
    def test_metadata_without_api_key(self, vectorstore):
        """Test metadata without API key."""
        metadata = vectorstore._get_metadata()
        assert len(metadata) == 0
    
    def test_get_digest(self, vectorstore):
        """Test getting collection digest."""
        digest = vectorstore.get_digest()
        
        assert "logical_clock" in digest
        assert "state_hash" in digest
        assert "buckets" in digest
        assert "count" in digest
        assert len(digest["buckets"]) == 256
    
    def test_delete(self, vectorstore):
        """Test deleting vectors."""
        result = vectorstore.delete(ids=["1", "2", "3"])
        assert result is True
    
    def test_delete_none(self, vectorstore):
        """Test delete with no IDs."""
        result = vectorstore.delete(ids=None)
        assert result is None


class TestHyperspaceVectorStoreIntegration:
    """Integration tests (require running server)."""
    
    @pytest.mark.integration
    def test_real_connection(self, mock_embeddings):
        """Test real connection to HyperspaceDB server.
        
        This test requires a running HyperspaceDB server on localhost:50051.
        Skip if server is not available.
        """
        try:
            vectorstore = HyperspaceVectorStore(
                host="localhost",
                port=50051,
                collection_name="integration_test",
                embedding_function=mock_embeddings,
            )
            
            # Try to add a text
            ids = vectorstore.add_texts(["Integration test"])
            assert len(ids) == 1
            
            # Try to search
            results = vectorstore.similarity_search("test", k=1)
            assert isinstance(results, list)
            
        except Exception as e:
            pytest.skip(f"HyperspaceDB server not available: {e}")



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from . import hyperspace_pb2 as hyperspace__pb2

GRPC_GENERATED_VERSION = '1.78.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + ' but the generated code in hyperspace_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class DatabaseStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateCollection = channel.unary_unary(
                '/hyperspace.Database/CreateCollection',
                request_serializer=hyperspace__pb2.CreateCollectionRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.StatusResponse.FromString,
                _registered_method=True)
        self.DeleteCollection = channel.unary_unary(
                '/hyperspace.Database/DeleteCollection',
                request_serializer=hyperspace__pb2.DeleteCollectionRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.StatusResponse.FromString,
                _registered_method=True)
        self.ListCollections = channel.unary_unary(
                '/hyperspace.Database/ListCollections',
                request_serializer=hyperspace__pb2.Empty.SerializeToString,
                response_deserializer=hyperspace__pb2.ListCollectionsResponse.FromString,
                _registered_method=True)
        self.GetCollectionStats = channel.unary_unary(
                '/hyperspace.Database/GetCollectionStats',
                request_serializer=hyperspace__pb2.CollectionStatsRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.CollectionStatsResponse.FromString,
                _registered_method=True)
        self.Insert = channel.unary_unary(
                '/hyperspace.Database/Insert',
                request_serializer=hyperspace__pb2.InsertRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.InsertResponse.FromString,
                _registered_method=True)
        self.BatchInsert = channel.unary_unary(
                '/hyperspace.Database/BatchInsert',
                request_serializer=hyperspace__pb2.BatchInsertRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.InsertResponse.FromString,
                _registered_method=True)
        self.InsertText = channel.unary_unary(
                '/hyperspace.Database/InsertText',
                request_serializer=hyperspace__pb2.InsertTextRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.InsertResponse.FromString,
                _registered_method=True)
        self.Vectorize = channel.unary_unary(
                '/hyperspace.Database/Vectorize',
                request_serializer=hyperspace__pb2.VectorizeRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.VectorizeResponse.FromString,
                _registered_method=True)
        self.SearchText = channel.unary_unary(
                '/hyperspace.Database/SearchText',
                request_serializer=hyperspace__pb2.SearchTextRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.SearchResponse.FromString,
                _registered_method=True)
        self.Delete = channel.unary_unary(
                '/hyperspace.Database/Delete',
                request_serializer=hyperspace__pb2.DeleteRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.DeleteResponse.FromString,
                _registered_method=True)
        self.Search = channel.unary_unary(
                '/hyperspace.Database/Search',
                request_serializer=hyperspace__pb2.SearchRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.SearchResponse.FromString,
                _registered_method=True)
        self.SearchBatch = channel.unary_unary(
                '/hyperspace.Database/SearchBatch',
                request_serializer=hyperspace__pb2.BatchSearchRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.BatchSearchResponse.FromString,
                _registered_method=True)
        self.SearchMultiCollection = channel.unary_unary(
                '/hyperspace.Database/SearchMultiCollection',
                request_serializer=hyperspace__pb2.SearchMultiCollectionRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.SearchMultiCollectionResponse.FromString,
                _registered_method=True)
        self.GetNode = channel.unary_unary(
                '/hyperspace.Database/GetNode',
                request_serializer=hyperspace__pb2.GetNodeRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.GraphNode.FromString,
                _registered_method=True)
        self.GetNeighbors = channel.unary_unary(
                '/hyperspace.Database/GetNeighbors',
                request_serializer=hyperspace__pb2.GetNeighborsRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.GetNeighborsResponse.FromString,
                _registered_method=True)
        self.GetConceptParents = channel.unary_unary(
                '/hyperspace.Database/GetConceptParents',
                request_serializer=hyperspace__pb2.GetConceptParentsRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.GetConceptParentsResponse.FromString,
                _registered_method=True)
        self.Traverse = channel.unary_unary(
                '/hyperspace.Database/Traverse',
                request_serializer=hyperspace__pb2.TraverseRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.TraverseResponse.FromString,
                _registered_method=True)
        self.FindSemanticClusters = channel.unary_unary(
                '/hyperspace.Database/FindSemanticClusters',
                request_serializer=hyperspace__pb2.FindSemanticClustersRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.FindSemanticClustersResponse.FromString,
                _registered_method=True)
        self.Monitor = channel.unary_stream(
                '/hyperspace.Database/Monitor',
                request_serializer=hyperspace__pb2.MonitorRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.SystemStats.FromString,
                _registered_method=True)
        self.TriggerSnapshot = channel.unary_unary(
                '/hyperspace.Database/TriggerSnapshot',
                request_serializer=hyperspace__pb2.Empty.SerializeToString,
                response_deserializer=hyperspace__pb2.StatusResponse.FromString,
                _registered_method=True)
        self.TriggerVacuum = channel.unary_unary(
                '/hyperspace.Database/TriggerVacuum',
                request_serializer=hyperspace__pb2.Empty.SerializeToString,
                response_deserializer=hyperspace__pb2.StatusResponse.FromString,
                _registered_method=True)
        self.TriggerReconsolidation = channel.unary_unary(
                '/hyperspace.Database/TriggerReconsolidation',
                request_serializer=hyperspace__pb2.ReconsolidationRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.StatusResponse.FromString,
                _registered_method=True)
        self.Configure = channel.unary_unary(
                '/hyperspace.Database/Configure',
                request_serializer=hyperspace__pb2.ConfigUpdate.SerializeToString,
                response_deserializer=hyperspace__pb2.StatusResponse.FromString,
                _registered_method=True)
        self.Replicate = channel.unary_stream(
                '/hyperspace.Database/Replicate',
                request_serializer=hyperspace__pb2.ReplicationRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.ReplicationLog.FromString,
                _registered_method=True)
        self.SubscribeToEvents = channel.unary_stream(
                '/hyperspace.Database/SubscribeToEvents',
                request_serializer=hyperspace__pb2.EventSubscriptionRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.EventMessage.FromString,
                _registered_method=True)
        self.GetDigest = channel.unary_unary(
                '/hyperspace.Database/GetDigest',
                request_serializer=hyperspace__pb2.DigestRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.DigestResponse.FromString,
                _registered_method=True)
        self.RebuildIndex = channel.unary_unary(
                '/hyperspace.Database/RebuildIndex',
                request_serializer=hyperspace__pb2.RebuildIndexRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.StatusResponse.FromString,
                _registered_method=True)
        self.SyncHandshake = channel.unary_unary(
                '/hyperspace.Database/SyncHandshake',
                request_serializer=hyperspace__pb2.SyncHandshakeRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.SyncHandshakeResponse.FromString,
                _registered_method=True)
        self.SyncPull = channel.unary_stream(
                '/hyperspace.Database/SyncPull',
                request_serializer=hyperspace__pb2.SyncPullRequest.SerializeToString,
                response_deserializer=hyperspace__pb2.SyncVectorData.FromString,
                _registered_method=True)
        self.SyncPush = channel.stream_unary(
                '/hyperspace.Database/SyncPush',
                request_serializer=hyperspace__pb2.SyncVectorData.SerializeToString,
                response_deserializer=hyperspace__pb2.SyncPushResponse.FromString,
                _registered_method=True)


class DatabaseServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CreateCollection(self, request, context):
        """Collection Management
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteCollection(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListCollections(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCollectionStats(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Insert(self, request, context):
        """Insert vectors
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BatchInsert(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def InsertText(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Vectorize(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SearchText(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Delete(self, request, context):
        """Delete vectors
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Search(self, request, context):
        """Search (ANN)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SearchBatch(self, request, context):
        """Batch Search (ANN)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SearchMultiCollection(self, request, context):
        """Multi-Geometry Search (v3.0)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetNode(self, request, context):
        """Graph Traversal API (v2.3)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetNeighbors(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetConceptParents(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Traverse(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FindSemanticClusters(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Monitor(self, request, context):
        """Stream statistics for TUI (Global or Collection tailored)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TriggerSnapshot(self, request, context):
        """Admin Controls
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TriggerVacuum(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TriggerReconsolidation(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Configure(self, request, context):
        """Dynamic Configuration
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Replicate(self, request, context):
        """Replication (Leader -> Follower)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribeToEvents(self, request, context):
        """CDC/Event Stream (External subscribers)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDigest(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RebuildIndex(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SyncHandshake(self, request, context):
        """Delta Sync (Merkle Tree — Task 2.1)
        Step 1: Client sends its digest, server returns which buckets differ.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SyncPull(self, request, context):
        """Step 2: Client requests vectors from specific buckets, server streams them.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SyncPush(self, request_iterator, context):
        """Step 3 (optional): Client pushes its unique vectors to the server.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DatabaseServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateCollection': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateCollection,
                    request_deserializer=hyperspace__pb2.CreateCollectionRequest.FromString,
                    response_serializer=hyperspace__pb2.StatusResponse.SerializeToString,
            ),
            'DeleteCollection': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteCollection,
                    request_deserializer=hyperspace__pb2.DeleteCollectionRequest.FromString,
                    response_serializer=hyperspace__pb2.StatusResponse.SerializeToString,
            ),
            'ListCollections': grpc.unary_unary_rpc_method_handler(
                    servicer.ListCollections,
                    request_deserializer=hyperspace__pb2.Empty.FromString,
                    response_serializer=hyperspace__pb2.ListCollectionsResponse.SerializeToString,
            ),
            'GetCollectionStats': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCollectionStats,
                    request_deserializer=hyperspace__pb2.CollectionStatsRequest.FromString,
                    response_serializer=hyperspace__pb2.CollectionStatsResponse.SerializeToString,
            ),
            'Insert': grpc.unary_unary_rpc_method_handler(
                    servicer.Insert,
                    request_deserializer=hyperspace__pb2.InsertRequest.FromString,
                    response_serializer=hyperspace__pb2.InsertResponse.SerializeToString,
            ),
            'BatchInsert': grpc.unary_unary_rpc_method_handler(
                    servicer.BatchInsert,
                    request_deserializer=hyperspace__pb2.BatchInsertRequest.FromString,
                    response_serializer=hyperspace__pb2.InsertResponse.SerializeToString,
            ),
            'InsertText': grpc.unary_unary_rpc_method_handler(
                    servicer.InsertText,
                    request_deserializer=hyperspace__pb2.InsertTextRequest.FromString,
                    response_serializer=hyperspace__pb2.InsertResponse.SerializeToString,
            ),
            'Vectorize': grpc.unary_unary_rpc_method_handler(
                    servicer.Vectorize,
                    request_deserializer=hyperspace__pb2.VectorizeRequest.FromString,
                    response_serializer=hyperspace__pb2.VectorizeResponse.SerializeToString,
            ),
            'SearchText': grpc.unary_unary_rpc_method_handler(
                    servicer.SearchText,
                    request_deserializer=hyperspace__pb2.SearchTextRequest.FromString,
                    response_serializer=hyperspace__pb2.SearchResponse.SerializeToString,
            ),
            'Delete': grpc.unary_unary_rpc_method_handler(
                    servicer.Delete,
                    request_deserializer=hyperspace__pb2.DeleteRequest.FromString,
                    response_serializer=hyperspace__pb2.DeleteResponse.SerializeToString,
            ),
            'Search': grpc.unary_unary_rpc_method_handler(
                    servicer.Search,
                    request_deserializer=hyperspace__pb2.SearchRequest.FromString,
                    response_serializer=hyperspace__pb2.SearchResponse.SerializeToString,
            ),
            'SearchBatch': grpc.unary_unary_rpc_method_handler(
                    servicer.SearchBatch,
                    request_deserializer=hyperspace__pb2.BatchSearchRequest.FromString,
                    response_serializer=hyperspace__pb2.BatchSearchResponse.SerializeToString,
            ),
            'SearchMultiCollection': grpc.unary_unary_rpc_method_handler(
                    servicer.SearchMultiCollection,
                    request_deserializer=hyperspace__pb2.SearchMultiCollectionRequest.FromString,
                    response_serializer=hyperspace__pb2.SearchMultiCollectionResponse.SerializeToString,
            ),
            'GetNode': grpc.unary_unary_rpc_method_handler(
                    servicer.GetNode,
                    request_deserializer=hyperspace__pb2.GetNodeRequest.FromString,
                    response_serializer=hyperspace__pb2.GraphNode.SerializeToString,
            ),
            'GetNeighbors': grpc.unary_unary_rpc_method_handler(
                    servicer.GetNeighbors,
                    request_deserializer=hyperspace__pb2.GetNeighborsRequest.FromString,
                    response_serializer=hyperspace__pb2.GetNeighborsResponse.SerializeToString,
            ),
            'GetConceptParents': grpc.unary_unary_rpc_method_handler(
                    servicer.GetConceptParents,
                    request_deserializer=hyperspace__pb2.GetConceptParentsRequest.FromString,
                    response_serializer=hyperspace__pb2.GetConceptParentsResponse.SerializeToString,
            ),
            'Traverse': grpc.unary_unary_rpc_method_handler(
                    servicer.Traverse,
                    request_deserializer=hyperspace__pb2.TraverseRequest.FromString,
                    response_serializer=hyperspace__pb2.TraverseResponse.SerializeToString,
            ),
            'FindSemanticClusters': grpc.unary_unary_rpc_method_handler(
                    servicer.FindSemanticClusters,
                    request_deserializer=hyperspace__pb2.FindSemanticClustersRequest.FromString,
                    response_serializer=hyperspace__pb2.FindSemanticClustersResponse.SerializeToString,
            ),
            'Monitor': grpc.unary_stream_rpc_method_handler(
                    servicer.Monitor,
                    request_deserializer=hyperspace__pb2.MonitorRequest.FromString,
                    response_serializer=hyperspace__pb2.SystemStats.SerializeToString,
            ),
            'TriggerSnapshot': grpc.unary_unary_rpc_method_handler(
                    servicer.TriggerSnapshot,
                    request_deserializer=hyperspace__pb2.Empty.FromString,
                    response_serializer=hyperspace__pb2.StatusResponse.SerializeToString,
            ),
            'TriggerVacuum': grpc.unary_unary_rpc_method_handler(
                    servicer.TriggerVacuum,
                    request_deserializer=hyperspace__pb2.Empty.FromString,
                    response_serializer=hyperspace__pb2.StatusResponse.SerializeToString,
            ),
            'TriggerReconsolidation': grpc.unary_unary_rpc_method_handler(
                    servicer.TriggerReconsolidation,
                    request_deserializer=hyperspace__pb2.ReconsolidationRequest.FromString,
                    response_serializer=hyperspace__pb2.StatusResponse.SerializeToString,
            ),
            'Configure': grpc.unary_unary_rpc_method_handler(
                    servicer.Configure,
                    request_deserializer=hyperspace__pb2.ConfigUpdate.FromString,
                    response_serializer=hyperspace__pb2.StatusResponse.SerializeToString,
            ),
            'Replicate': grpc.unary_stream_rpc_method_handler(
                    servicer.Replicate,
                    request_deserializer=hyperspace__pb2.ReplicationRequest.FromString,
                    response_serializer=hyperspace__pb2.ReplicationLog.SerializeToString,
            ),
            'SubscribeToEvents': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribeToEvents,
                    request_deserializer=hyperspace__pb2.EventSubscriptionRequest.FromString,
                    response_serializer=hyperspace__pb2.EventMessage.SerializeToString,
            ),
            'GetDigest': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDigest,
                    request_deserializer=hyperspace__pb2.DigestRequest.FromString,
                    response_serializer=hyperspace__pb2.DigestResponse.SerializeToString,
            ),
            'RebuildIndex': grpc.unary_unary_rpc_method_handler(
                    servicer.RebuildIndex,
                    request_deserializer=hyperspace__pb2.RebuildIndexRequest.FromString,
                    response_serializer=hyperspace__pb2.StatusResponse.SerializeToString,
            ),
            'SyncHandshake': grpc.unary_unary_rpc_method_handler(
                    servicer.SyncHandshake,
                    request_deserializer=hyperspace__pb2.SyncHandshakeRequest.FromString,
                    response_serializer=hyperspace__pb2.SyncHandshakeResponse.SerializeToString,
            ),
            'SyncPull': grpc.unary_stream_rpc_method_handler(
                    servicer.SyncPull,
                    request_deserializer=hyperspace__pb2.SyncPullRequest.FromString,
                    response_serializer=hyperspace__pb2.SyncVectorData.SerializeToString,
            ),
            'SyncPush': grpc.stream_unary_rpc_method_handler(
                    servicer.SyncPush,
                    request_deserializer=hyperspace__pb2.SyncVectorData.FromString,
                    response_serializer=hyperspace__pb2.SyncPushResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'hyperspace.Database', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('hyperspace.Database', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class Database(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CreateCollection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/CreateCollection',
            hyperspace__pb2.CreateCollectionRequest.SerializeToString,
            hyperspace__pb2.StatusResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteCollection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/DeleteCollection',
            hyperspace__pb2.DeleteCollectionRequest.SerializeToString,
            hyperspace__pb2.StatusResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListCollections(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/ListCollections',
            hyperspace__pb2.Empty.SerializeToString,
            hyperspace__pb2.ListCollectionsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetCollectionStats(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/GetCollectionStats',
            hyperspace__pb2.CollectionStatsRequest.SerializeToString,
            hyperspace__pb2.CollectionStatsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Insert(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/Insert',
            hyperspace__pb2.InsertRequest.SerializeToString,
            hyperspace__pb2.InsertResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def BatchInsert(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/BatchInsert',
            hyperspace__pb2.BatchInsertRequest.SerializeToString,
            hyperspace__pb2.InsertResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def InsertText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/InsertText',
            hyperspace__pb2.InsertTextRequest.SerializeToString,
            hyperspace__pb2.InsertResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Vectorize(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/Vectorize',
            hyperspace__pb2.VectorizeRequest.SerializeToString,
            hyperspace__pb2.VectorizeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SearchText(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/SearchText',
            hyperspace__pb2.SearchTextRequest.SerializeToString,
            hyperspace__pb2.SearchResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Delete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/Delete',
            hyperspace__pb2.DeleteRequest.SerializeToString,
            hyperspace__pb2.DeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Search(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/Search',
            hyperspace__pb2.SearchRequest.SerializeToString,
            hyperspace__pb2.SearchResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SearchBatch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/SearchBatch',
            hyperspace__pb2.BatchSearchRequest.SerializeToString,
            hyperspace__pb2.BatchSearchResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SearchMultiCollection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/SearchMultiCollection',
            hyperspace__pb2.SearchMultiCollectionRequest.SerializeToString,
            hyperspace__pb2.SearchMultiCollectionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetNode(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/GetNode',
            hyperspace__pb2.GetNodeRequest.SerializeToString,
            hyperspace__pb2.GraphNode.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetNeighbors(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/GetNeighbors',
            hyperspace__pb2.GetNeighborsRequest.SerializeToString,
            hyperspace__pb2.GetNeighborsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetConceptParents(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/GetConceptParents',
            hyperspace__pb2.GetConceptParentsRequest.SerializeToString,
            hyperspace__pb2.GetConceptParentsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Traverse(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/Traverse',
            hyperspace__pb2.TraverseRequest.SerializeToString,
            hyperspace__pb2.TraverseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def FindSemanticClusters(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/FindSemanticClusters',
            hyperspace__pb2.FindSemanticClustersRequest.SerializeToString,
            hyperspace__pb2.FindSemanticClustersResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Monitor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/hyperspace.Database/Monitor',
            hyperspace__pb2.MonitorRequest.SerializeToString,
            hyperspace__pb2.SystemStats.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def TriggerSnapshot(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/TriggerSnapshot',
            hyperspace__pb2.Empty.SerializeToString,
            hyperspace__pb2.StatusResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def TriggerVacuum(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/TriggerVacuum',
            hyperspace__pb2.Empty.SerializeToString,
            hyperspace__pb2.StatusResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def TriggerReconsolidation(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/TriggerReconsolidation',
            hyperspace__pb2.ReconsolidationRequest.SerializeToString,
            hyperspace__pb2.StatusResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Configure(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/Configure',
            hyperspace__pb2.ConfigUpdate.SerializeToString,
            hyperspace__pb2.StatusResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Replicate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/hyperspace.Database/Replicate',
            hyperspace__pb2.ReplicationRequest.SerializeToString,
            hyperspace__pb2.ReplicationLog.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SubscribeToEvents(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/hyperspace.Database/SubscribeToEvents',
            hyperspace__pb2.EventSubscriptionRequest.SerializeToString,
            hyperspace__pb2.EventMessage.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetDigest(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/GetDigest',
            hyperspace__pb2.DigestRequest.SerializeToString,
            hyperspace__pb2.DigestResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def RebuildIndex(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/RebuildIndex',
            hyperspace__pb2.RebuildIndexRequest.SerializeToString,
            hyperspace__pb2.StatusResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SyncHandshake(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/hyperspace.Database/SyncHandshake',
            hyperspace__pb2.SyncHandshakeRequest.SerializeToString,
            hyperspace__pb2.SyncHandshakeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SyncPull(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/hyperspace.Database/SyncPull',
            hyperspace__pb2.SyncPullRequest.SerializeToString,
            hyperspace__pb2.SyncVectorData.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SyncPush(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(
            request_iterator,
            target,
            '/hyperspace.Database/SyncPush',
            hyperspace__pb2.SyncVectorData.SerializeToString,
            hyperspace__pb2.SyncPushResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



"""HyperspaceDB VectorStore implementation for LangChain."""

from __future__ import annotations

import hashlib
import logging
from typing import Any, Callable, Iterable, List, Optional, Tuple, Type

import grpc
import numpy as np
from langchain_core.documents import Document
from langchain_core.embeddings import Embeddings
from langchain_core.vectorstores import VectorStore

logger = logging.getLogger(__name__)

class HyperspaceVectorStore(VectorStore):
    """HyperspaceDB vector store integration for LangChain."""

    def __init__(
        self,
        host: str = "localhost",
        port: int = 50051,
        collection_name: str = "default",
        embedding_function: Optional[Embeddings] = None,
        api_key: Optional[str] = None,
        user_id: Optional[str] = None,
        dimension: int = 1536,
        metric: str = "l2",
        enable_deduplication: bool = True,
        use_server_side_embedding: bool = False,
        **kwargs: Any,
    ):
        from .client import HyperspaceClient
        self.host = host
        self.port = port
        self.collection_name = collection_name
        self._embedding_function = embedding_function
        self.api_key = api_key
        self.user_id = user_id
        self.dimension = dimension
        self.metric = metric
        self.enable_deduplication = enable_deduplication
        self.use_server_side_embedding = use_server_side_embedding
        
        self._client = HyperspaceClient(
            host=f"{host}:{port}",
            api_key=api_key,
            user_id=user_id,
        )
        
        if not use_server_side_embedding and embedding_function is None:
            raise ValueError("embedding_function is required when use_server_side_embedding is False")

        self._ensure_collection()
        logger.info(f"Initialized HyperspaceVectorStore: {host}:{port}/{collection_name}")

    def _ensure_collection(self) -> None:
        try:
            self._client.create_collection(
                self.collection_name, 
                dimension=self.dimension, 
                metric=self.metric
            )
        except Exception as e:
            logger.debug(f"Collection creation skipped: {e}")

    def _compute_content_hash(self, text: str) -> int:
        hash_bytes = hashlib.sha256(text.encode("utf-8")).digest()
        return int.from_bytes(hash_bytes[:4], byteorder="big")

    @property
    def embeddings(self) -> Optional[Embeddings]:
        return self._embedding_function

    def add_texts(
        self,
        texts: Iterable[str],
        metadatas: Optional[List[dict]] = None,
        ids: Optional[List[int]] = None,
        **kwargs: Any,
    ) -> List[str]:
        texts_list = list(texts)
        if not texts_list:
            return []
        
        if metadatas is None:
            metadatas = [{} for _ in texts_list]
        
        for i, text in enumerate(texts_list):
            metadatas[i]["text"] = text
        
        if ids is None:
            if self.enable_deduplication:
                ids = [self._compute_content_hash(text) for text in texts_list]
            else:
                import time, random
                ids = [int(time.time() * 1000) + random.randint(0, 1000) for _ in texts_list]
        
        try:
            if self.use_server_side_embedding:
                for i, text in enumerate(texts_list):
                    self._client.insert_text(
                        id=ids[i],
                        text=text,
                        collection=self.collection_name,
                        metadata={str(k): str(v) for k, v in metadatas[i].items()}
                    )
            else:
                if self._embedding_function is None:
                    raise ValueError("Embedding function is required")
                embeddings_data = self._embedding_function.embed_documents(texts_list)
                self._client.batch_insert(
                    vectors=embeddings_data,
                    ids=ids,
                    metadatas=[{str(k): str(v) for k, v in m.items()} for m in metadatas],
                    collection=self.collection_name,
                )
            return [str(x) for x in ids]
        except Exception as e:
            logger.error(f"Failed to batch insert vectors: {e}")
            raise

    def similarity_search_with_score(
        self,
        query: str,
        k: int = 4,
        filter: Optional[dict] = None,
        **kwargs: Any,
    ) -> List[Tuple[Document, float]]:
        if self.use_server_side_embedding:
            hits = self._client.search_text(text=query, top_k=k, collection=self.collection_name)
        else:
            if self._embedding_function is None:
                raise ValueError("Embedding function is required")
            embedding = self._embedding_function.embed_query(query)
            hits = self._client.search(vector=embedding, top_k=k, collection=self.collection_name)
        
        return self._parse_hits(hits)

    def _parse_hits(self, hits: List[Any]) -> List[Tuple[Document, float]]:
        results = []
        for hit in hits:
            metadata = getattr(hit, "metadata", {}) if not isinstance(hit, dict) else hit.get("metadata", {})
            distance = getattr(hit, "distance", 0.0) if not isinstance(hit, dict) else hit.get("distance", 0.0)
            text = metadata.get("text", "")
            doc = Document(page_content=text, metadata=dict(metadata))
            results.append((doc, float(distance)))
        return results

    def similarity_search(
        self,
        query: str,
        k: int = 4,
        filter: Optional[dict] = None,
        **kwargs: Any,
    ) -> List[Document]:
        docs_and_scores = self.similarity_search_with_score(query, k=k, filter=filter, **kwargs)
        return [doc for doc, _ in docs_and_scores]

    def delete(self, ids: Optional[List[str]] = None, **kwargs: Any) -> Optional[bool]:
        if ids is None:
            return None
        success = True
        for id_str in ids:
            try:
                id_num = int(id_str) if id_str.isdigit() else self._compute_content_hash(id_str)
                self._client.delete(id=id_num, collection=self.collection_name)
            except Exception as e:
                logger.error(f"Failed to delete {id_str}: {e}")
                success = False
        return success

    def max_marginal_relevance_search(
        self,
        query: str,
        k: int = 4,
        fetch_k: int = 20,
        lambda_mult: float = 0.5,
        **kwargs: Any,
    ) -> List[Document]:
        if self.use_server_side_embedding:
            return self.similarity_search(query, k=k, **kwargs)
        return super().max_marginal_relevance_search(query, k=k, fetch_k=fetch_k, lambda_mult=lambda_mult, **kwargs)

    def __del__(self) -> None:
        if hasattr(self, "_client"):
            self._client.close()

    @classmethod
    def from_texts(
        cls: Type[HyperspaceVectorStore],
        texts: List[str],
        embedding: Embeddings,
        metadatas: Optional[List[dict]] = None,
        **kwargs: Any,
    ) -> HyperspaceVectorStore:
        vectorstore = cls(embedding_function=embedding, **kwargs)
        vectorstore.add_texts(texts, metadatas=metadatas)
        return vectorstore

    @classmethod
    def from_documents(
        cls: Type[HyperspaceVectorStore],
        documents: List[Document],
        embedding: Embeddings,
        **kwargs: Any,
    ) -> HyperspaceVectorStore:
        texts = [doc.page_content for doc in documents]
        metadatas = [doc.metadata for doc in documents]
        return cls.from_texts(texts, embedding, metadatas=metadatas, **kwargs)



"""HyperspaceDB Python gRPC client.

This module provides a simple Python client for HyperspaceDB using gRPC.
It is used by the LangChain integration.
"""

from typing import Dict, List, Optional, Tuple, Any
import grpc
try:
    from .generated import hyperspace_pb2, hyperspace_pb2_grpc
except ImportError:
    # Fallback for direct scripts
    from langchain_hyperspace.generated import hyperspace_pb2, hyperspace_pb2_grpc


class HyperspaceClient:
    """Simple Python client for HyperspaceDB.
    
    This is a lightweight wrapper around the gRPC API.
    """
    
    def __init__(
        self,
        host: str = "localhost",
        port: int = 50051,
        api_key: Optional[str] = None,
        user_id: Optional[str] = None,
    ):
        target = f"{host}:{port}" if ":" not in host else host
        self._channel = grpc.insecure_channel(target)
        self._stub = hyperspace_pb2_grpc.DatabaseStub(self._channel)
        self.api_key = api_key
        self.user_id = user_id
        
    def _get_metadata(self) -> List[Tuple[str, str]]:
        meta = []
        if self.api_key:
            meta.append(("x-api-key", self.api_key))
        if self.user_id:
            meta.append(("x-hyperspace-user-id", self.user_id))
        return meta
    
    def create_collection(self, name: str, dimension: int, metric: str = "l2") -> bool:
        req = hyperspace_pb2.CreateCollectionRequest(name=name, dimension=dimension, metric=metric)
        self._stub.CreateCollection(req, metadata=self._get_metadata())
        return True
    
    def insert(self, collection: str, id: int, vector: List[float], metadata: Dict[str, str]) -> bool:
        req = hyperspace_pb2.InsertRequest(collection=collection, id=id, vector=vector, metadata=metadata)
        res = self._stub.Insert(req, metadata=self._get_metadata())
        return res.success

    def insert_text(self, id: int, text: str, collection: str, metadata: Dict[str, str] = None) -> bool:
        req = hyperspace_pb2.InsertTextRequest(collection=collection, id=id, text=text, metadata=metadata or {})
        res = self._stub.InsertText(req, metadata=self._get_metadata())
        return res.success

    def batch_insert(self, vectors: List[List[float]], ids: List[int], metadatas: List[Dict[str, str]], collection: str) -> bool:
        items = []
        for i in range(len(vectors)):
            vdata = hyperspace_pb2.VectorData(id=ids[i], vector=vectors[i], metadata=metadatas[i])
            items.append(vdata)
        req = hyperspace_pb2.BatchInsertRequest(collection=collection, vectors=items)
        res = self._stub.BatchInsert(req, metadata=self._get_metadata())
        return res.success

    def search(self, vector: List[float], top_k: int, collection: str, filters: List[Any] = None) -> List[Any]:
        req = hyperspace_pb2.SearchRequest(collection=collection, vector=vector, top_k=top_k)
        if filters:
             # Logic to add filters to the req
             pass
        res = self._stub.Search(req, metadata=self._get_metadata())
        return res.results

    def search_text(self, text: str, top_k: int, collection: str) -> List[Any]:
        req = hyperspace_pb2.SearchTextRequest(collection=collection, text=text, top_k=top_k)
        res = self._stub.SearchText(req, metadata=self._get_metadata())
        return res.results

    def get_digest(self, collection: str) -> Dict[str, Any]:
        req = hyperspace_pb2.DigestRequest(collection=collection)
        res = self._stub.GetDigest(req, metadata=self._get_metadata())
        return {"count": res.count, "logical_clock": res.logical_clock}

    def delete(self, collection: str, id: int) -> bool:
        req = hyperspace_pb2.DeleteRequest(collection=collection, id=id)
        res = self._stub.Delete(req, metadata=self._get_metadata())
        return res.success

    def close(self):
        self._channel.close()
